-- (c) M.P.I 98/99 -- Date: 98.12.03

module BTree where

import Mpi
import List

-- (1) Datatype definition -----------------------------------------------------

data BTree a = Empty | Node(a, (BTree a, BTree a)) deriving Show

inBTree = either (const Empty) Node

outBTree :: BTree a -> Either () (a,(BTree a,BTree a))
outBTree Empty              = Left ()
outBTree (Node (a,(t1,t2))) = Right(a,(t1,t2))

-- (2) Ana + cata + hylo -------------------------------------------------------

cataBTree g = g . (recBTree (cataBTree g)) . outBTree

anaBTree g = inBTree . (recBTree (anaBTree g) ) . g

hyloBTree h g = cataBTree h . anaBTree g

recBTree f = id -|- (id >< (f >< f))

-- (3) Map ---------------------------------------------------------------------

instance Functor BTree
         where map f = cataBTree ( inBTree . (id -|- (f >< (id >< id))) )

-- (4) Examples ----------------------------------------------------------------

-- (4.1) Quicksort -------------------------------------------------------------

qSort :: Ord a => [a] -> [a]
qSort = hyloBTree (either (const []) join) qsep

-- where

join(x,(l,r))=l++[x]++r

qsep []    = Left ()
qsep (h:t) = Right (h,(s,l)) where (s,l) = part (<h) t

part p []                = ([],[])
part p (h:t) | p h       = let (s,l) = part p t in (h:s,l)
             | otherwise = let (s,l) = part p t in (s,h:l)

-- (4.2) Traces ----------------------------------------------------------------

traces :: Eq a => BTree a -> [[a]]
traces = cataBTree (either (const [[]]) put)

-- where

put(a,(l,r)) = union (map (a:) l) (map (a:) r) 

-- (4.3) Towers of Hanoi -------------------------------------------------------

hanoi = hyloBTree present strategy

--- where

present = either (const []) join

strategy (0,a,b,c) = Left ()
strategy (n,a,b,c) = Right (a++"->"++c, ((n-1,a,c,b),(n-1,b,a,c)))

--------------------------------------------------------------------------------

